
   .file        "commonIRQ.S"

  .text
  .section .text
  .code 32

@******************************************************************************
@                            CONSTANT DECLARATIONS
@******************************************************************************

#define NO_INT   0xC0
#define MODE_SYS 0x1F
#define MODE_IRQ 0x12


@******************************************************************************
@                            EXTERNAL DECLARATIONS
@******************************************************************************
  .extern  handleIRQs


@******************************************************************************
@ Description:
@   A general interrupt handler.
@   Save the current context on the stack and switch mode to: SYSTEM.
@******************************************************************************
        .align 2
        .global _generalIrqHandler
        .type _generalIrqHandler,function

_generalIrqHandler:
        SUB     LR,LR,#4
        STMFD   SP!,{R0-R12,LR}     //store registers
        MRS     R1,SPSR
        STMFD   SP!,{R1}

@        STMFD   SP,{LR}^
@        SUB     SP,SP,#4

        @
        @ Disable interrupts and change to SYSTEM mode (from IRQ mode)
        @
        MSR     CPSR_c,#(NO_INT | MODE_SYS)
        
        @
        @ Handle the IRQ by calling the appropriate ISR
        @ (that will execute in SYSTEM mode)
        @
        BL      handleIRQs

        @
        @ Change to IRQ mode (from SYSTEM mode)
        @
        MSR     CPSR_c,#(NO_INT | MODE_IRQ)

@        LDMFD   SP,{LR}^
@        ADD     SP,SP,#4

        @
        @ Restore interrupted context and return
        @
        LDMFD   SP!,{R1}
        MSR     SPSR_cxsf,R1
        LDMFD   SP!,{R0-R12,PC}^   //Restore registers and return
